/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test TestPrintGCDetails
 * @bug 8035406 8027295
 * @summary Ensure that the PrintGCDetails output for a minor GC with G1
 * includes the expected necessary messages.
 * @key gc
 * @library /testlibrary
 */

import com.oracle.java.testlibrary.ProcessTools;
import com.oracle.java.testlibrary.OutputAnalyzer;

public class TestGCLogMessages {
  public static void main(String[] args) throws Exception {

    ProcessBuilder pb = ProcessTools.createJavaProcessBuilder("-XX:+UseG1GC",
                                                              "-Xmx10M",
                                                              GCTest.class.getName());

    OutputAnalyzer output = new OutputAnalyzer(pb.start());

    output.shouldNotContain("[Code Root Purge");
    output.shouldNotContain("[Young Free CSet");
    output.shouldNotContain("[Non-Young Free CSet");
    output.shouldHaveExitValue(0);

    pb = ProcessTools.createJavaProcessBuilder("-XX:+UseG1GC",
                                               "-Xmx10M",
                                               "-XX:+PrintGCDetails",
                                               GCTest.class.getName());

    output = new OutputAnalyzer(pb.start());

    output.shouldContain("[Code Root Purge");
    output.shouldNotContain("[Young Free CSet");
    output.shouldNotContain("[Non-Young Free CSet");
    output.shouldHaveExitValue(0);

    pb = ProcessTools.createJavaProcessBuilder("-XX:+UseG1GC",
                                               "-Xmx10M",
                                               "-XX:+PrintGCDetails",
                                               "-XX:+UnlockExperimentalVMOptions",
                                               "-XX:G1LogLevel=finest",
                                               GCTest.class.getName());

    output = new OutputAnalyzer(pb.start());

    output.shouldContain("[Code Root Purge");
    output.shouldContain("[Young Free CSet");
    output.shouldContain("[Non-Young Free CSet");
    output.shouldHaveExitValue(0);

  }

  static class GCTest {
    private static byte[] garbage;
    public static void main(String [] args) {
      System.out.println("Creating garbage");
      // create 128MB of garbage. This should result in at least one GC
      for (int i = 0; i < 1024; i++) {
        garbage = new byte[128 * 1024];
      }
      System.out.println("Done");
    }
  }
}
